<?php
/**
 * @brief  planner 모듈의 admin view class
 * @author BNU <bnufactory@gmail.com>
 **/
class plannerAdminView extends planner {

    function init() {
        parent::init();

        // module_srl이 있으면 미리 체크하여 존재하는 모듈이면 module_info 세팅
        $module_srl = Context::get('module_srl');
        if(!$module_srl && $this->module_srl) {
            $module_srl = $this->module_srl;
            Context::set('module_srl', $module_srl);
        }

        // module model 객체 생성
        $oModuleModel = &getModel('module');

        // module_srl이 넘어오면 해당 모듈의 정보를 미리 구해 놓음
        if($module_srl) {
            $module_info = $oModuleModel->getModuleInfoByModuleSrl($module_srl);
            if(!$module_info) {
                Context::set('module_srl','');
                $this->act = 'list';
            } else {
                $this->module_info = $module_info;
                Context::set('module_info', $module_info);
            }
        }

        // 템플릿 경로 지정
        $template_path = sprintf('%stpl/', $this->module_path);
        $this->setTemplatePath($template_path);
    }


    function dispPlannerAdminContent() {
        Context::set('config', $this->module_config);

        $args->sort_index = 'module_srl';
        $args->page = Context::get('page');
        $args->s_module_category_srl = Context::get('module_category_srl');
        $output = executeQuery('planner.getPlannerList', $args);

        Context::set('planner_list', $output->data);


        // 스킨 목록을 구해옴
        $oModuleModel = &getModel('module');
        $skin_list = $oModuleModel->getSkins($this->module_path);
        Context::set('skin_list', $skin_list);

        $this->setTemplateFile('index');
    }


    function dispPlannerAdminPlannerInfo() {
        // module_srl 값이 없다면 그냥 index 페이지를 보여줌
        if(!Context::get('module_srl')) return $this->dispBoardAdminContent();

        // 레이아웃이 정해져 있다면 레이아웃 정보를 추가해줌(layout_title, layout)
        if($this->module_info->layout_srl) {
            $oLayoutModel = &getModel('layout');
            $layout_info = $oLayoutModel->getLayout($this->module_info->layout_srl);
            $this->module_info->layout = $layout_info->layout;
            $this->module_info->layout_title = $layout_info->layout_title;
        }

        // 정해진 스킨이 있으면 해당 스킨의 정보를 구함
        if($this->module_info->skin) {
            $oModuleModel = &getModel('module');
            $skin_info = $oModuleModel->loadSkinInfo($this->module_path, $this->module_info->skin);
            $this->module_info->skin_title = $skin_info->title;
        }

    	$this->setTemplateFile('planner_info');
    }

    
    
    function dispPlannerAdminInsertPlanner() {
        // 스킨 목록을 구해옴
        $oModuleModel = &getModel('module');
        $skin_list = $oModuleModel->getSkins($this->module_path);
        Context::set('skin_list',$skin_list);

        // 레이아웃 목록을 구해옴
        $oLayoutMode = &getModel('layout');
        $layout_list = $oLayoutMode->getLayoutList();
        Context::set('layout_list', $layout_list);

    	$this->setTemplateFile('planner_insert');
    }
    
    
    function dispPlannerAdminSkinInfo() {
        // 현재 선택된 모듈의 스킨의 정보 xml 파일을 읽음
        $module_info = Context::get('module_info');
        $skin = $module_info->skin;

        $oModuleModel = &getModel('module');
        $skin_info = $oModuleModel->loadSkinInfo($this->module_path, $skin);

        // skin_info에 extra_vars 값을 지정
        if(count($skin_info->extra_vars)) {
            foreach($skin_info->extra_vars as $key => $val) {
                $group = $val->group;
                $name = $val->name;
                $type = $val->type;
                $value = $module_info->{$name};
                if($type=="checkbox"&&!$value) $value = array();
                $skin_info->extra_vars[$key]->value= $value;
            }
        }

        Context::set('skin_info', $skin_info);
        $this->setTemplateFile('skin_info');
    }
    
    
    function dispPlannerAdminCategoryInfo() {
        // module_srl을 구함
        $module_srl = $this->module_info->module_srl;

        // 카테고리 정보를 가져옴
        $oPlannerModel = &getModel('planner');
        $category_xml_file = $oPlannerModel->getCategoryXmlFile($module_srl);


        $this->setTemplateFile('category_list');
    }

    function dispPlannerAdminInsertModule() {
        $this->setTemplateFile('insert_module');
    }

}
?>