<?php
/**
 * @brief  planner 모듈의 Controller class
 * @author BNU <bnufactory@gmail.com>
 **/
class plannerController extends planner {

	function init() {}

	function procPlannerAddEntry() {
		$obj = Context::getRequestVars();

		$oDB = &DB::getInstance();
		$oDB->begin();

		// 주어진 문서 번호가 없으면 문서 번호 등록
		if(!$obj->entry_srl) $obj->entry_srl = getNextSequence();

		$entry->entry_srl = $obj->entry_srl;
		$entry->parent_srl = $obj->parent_srl;
		$entry->type = (!$obj->type) ? 'schedule' : $obj->type;
		$entry->title = $obj->title;
		$entry->description = $obj->description;

		$begin_date['year'] = sprintf('%04d', $obj->begin_year);
		$begin_date['month'] = sprintf('%02d', $obj->begin_month);
		$begin_date['day'] = sprintf('%02d', $obj->begin_day);
		$begin_date['time'] = str_replace(':', '', $obj->begin_time);
		$end_date['year'] = sprintf('%04d', $obj->begin_year);
		$end_date['month'] = sprintf('%02d', $obj->begin_month);
		$end_date['day'] = sprintf('%02d', $obj->begin_day);
		$end_date['time'] = str_replace(':', '', $obj->begin_time);
		$entry->begin_date = implode('', $begin_date);
		$entry->end_date = implode('', $end_date);


		$output = executeQuery('planner.insertentry', $entry);
		if(!$output->toBool()) { $oDB->rollback(); return $output; }

		$oPlannerModel = &getModel('planner');
		$entries = $oPlannerModel->getEntriesJSON($entry->entry_srl);
		$this->add('entries', $entries);
		return $output;
	}


	/**
	 * @brief 카테고리를 캐시 파일로 저장
	 **/
	function makeCategoryFile($module_srl) {
		// 캐시 파일 생성시 필요한 정보가 없으면 그냥 return
		if(!$module_srl) return false;

		// 모듈 정보를 가져옴 (mid를 구하기 위해)
		$oModuleModel = &getModel('module');
		$module_info = $oModuleModel->getModuleInfoByModuleSrl($module_srl);
		$mid = $module_info->mid;

		if(!is_dir($this->category_cache_path)) FileHandler::makeDir($this->category_cache_path);

		// 캐시 파일의 이름을 지정
		$xml_file = sprintf($this->category_cache_path.'category_%s.xml.php', $module_srl);
		$php_file = sprintf($this->category_cache_path.'category_%s.php', $module_srl);

		// 카테고리 목록을 구함
		$args->target_srl = $module_srl;
		$output = executeQueryArray('planner.getCategoryList', $args);

		// 반환목록이 없으면 리턴
		if(!$output->data) {
			FileHandler::removeFile($xml_file);
			FileHandler::removeFile($php_file);
			return false;
		}

		$category_count = count($output->data);
		for($i = 0; $i < $category_count; $i++) {
			$category_srl = $output->data[$i]->category_srl;
			if(!preg_match('/^[0-9,]+$/', $output->data[$i]->group_srls)) $output->data[$i]->group_srls = '';
			$category_list[$category_srl] = $output->data[$i];
		}

		// 구해온 데이터가 없다면 노드데이터가 없는 xml 파일만 생성
		if(!$category_list) {
			$xml_buff = "<root />";
			FileHandler::writeFile($xml_file, $xml_buff);
			FileHandler::writeFile($php_file, '<?php if(!defined("__ZBXE__")) exit(); ?>');
			return $xml_file;
		}

		// 루프를 돌면서 tree 구성
		foreach($category_list as $category_srl => $node) {
			$node->mid = $mid;
			$parent_srl = (int)$node->parent_srl;
			$tree[$parent_srl][$category_srl] = $node;
		}

		// 캐시 파일의 권한과 그룹 설정을 위한 공통 헤더
		$header_script =
			'$lang_type = Context::getLangType(); '.
			'$is_logged = Context::get(\'is_logged\'); '.
			'$logged_info = Context::get(\'logged_info\'); '.
			'if($is_logged) {'.
				'if($logged_info->is_admin=="Y") $is_admin = true; '.
				'else $is_admin = false; '.
				'$group_srls = array_keys($logged_info->group_list); '.
			'} else { '.
				'$is_admin = false; '.
				'$group_srsl = array(); '.
			'} ';

		// xml 캐시 파일 생성 (xml캐시는 따로 동작하기에 session 지정을 해주어야 함)
		$xml_buff = sprintf(
			'<?php '.
			'define(\'__ZBXE__\', true); '.
			'require_once(\'../../../config/config.inc.php\'); '.
			'$oContext = &Context::getInstance(); '.
			'$oContext->init(); '.
			'header("Content-Type: text/xml; charset=UTF-8"); '.
			'header("Expires: Mon, 26 Jul 1997 05:00:00 GMT"); '.
			'header("Last-Modified: " . gmdate("D, d M Y H:i:s") . " GMT"); '.
			'header("Cache-Control: no-store, no-cache, must-revalidate"); '.
			'header("Cache-Control: post-check=0, pre-check=0", false); '.
			'header("Pragma: no-cache"); '.
			'%s'.
			'?>'.
			'<root>%s</root>',
			$header_script,
			$this->getXmlTree($tree[0], $tree)
		);

		// php 캐시 파일 생성
		$php_output = $this->getPhpCacheCode($tree[0], $tree);
		$php_buff = sprintf(
			'<?php '.
			'if(!defined("__ZBXE__")) exit(); '.
			'%s; '.
			'%s; '.
			'$menu->list = array(%s); '.
			'?>',
			$header_script,
			$php_output['category_title_str'],
			$php_output['buff']
		);

		// 파일 저장
		FileHandler::writeFile($xml_file, $xml_buff);
		FileHandler::writeFile($php_file, $php_buff);
		return $xml_file;
	}

	/**
	 * @brief array로 정렬된 노드들을 parent_srl을 참조하면서 recursive하게 돌면서 xml 데이터 생성
	 * 메뉴 xml파일은 node라는 tag가 중첩으로 사용되며 이 xml doc으로 관리자 페이지에서 메뉴를 구성해줌\n
	 * (tree_menu.js 에서 xml파일을 바로 읽고 tree menu를 구현)
	 **/
	function getXmlTree($source_node, $tree) {
		if(!$source_node) return;

		foreach($source_node as $category_srl => $node) {
			$child_buff = "";

			// 자식 노드의 데이터 가져옴
			if($category_srl && $tree[$category_srl]) $child_buff = $this->getXmlTree($tree[$category_srl], $tree);

			// 변수 정리
			$title = str_replace(array('&','"','<','>'),array('&amp;','&quot;','&lt;','&gt;'),$node->title);
			$expand = $node->expand;
			$group_srls = $node->group_srls;
			$mid = $node->mid;
			$target_srl = $node->target_srl;
			$color = $node->color;

			// node->group_srls값이 있으면
			if($group_srls) $group_check_code = sprintf('($is_admin==true||(is_array($group_srls)&&count(array_intersect($group_srls, array(%s)))))',$group_srls);
			else $group_check_code = "true";

			$attribute = sprintf(
					'mid="%s" target_srl="%d" node_srl="%d" category_srl="%d" color="%s" text="<?php echo (%s?"%s":"")?>" url="%s" expand="%s" document_count="%d"',
					$mid,
					$target_srl,
					$category_srl,
					$category_srl,
					$color,
					$group_check_code,
					$title,
					getUrl('', 'mid', $node->mid, 'category', $category_srl),
					$expand,
					$node->document_count
			);

			if($child_buff) $buff .= sprintf('<node %s>%s</node>', $attribute, $child_buff);
			else $buff .=  sprintf('<node %s />', $attribute);
		}
		return $buff;
	}

	/**
	 * @brief array로 정렬된 노드들을 php code로 변경하여 return
	 * 메뉴에서 메뉴를 tpl에 사용시 xml데이터를 사용할 수도 있지만 별도의 javascript 사용이 필요하기에
	 * php로 된 캐시파일을 만들어서 db이용없이 바로 메뉴 정보를 구할 수 있도록 한다
	 * 이 캐시는 ModuleHandler::displayContent() 에서 include하여 Context::set() 한다
	 **/
	function getPhpCacheCode($source_node, $tree) {
		$output = array("buff"=>"", "category_srl_list"=>array());
		if(!$source_node) return $output;

		// 루프를 돌면서 1차 배열로 정리하고 include할 수 있는 php script 코드를 생성
		foreach($source_node as $category_srl => $node) {

			// 자식 노드가 있으면 자식 노드의 데이터를 먼저 얻어옴
			if($category_srl&&$tree[$category_srl]) $child_output = $this->getPhpCacheCode($tree[$category_srl], $tree);
			else $child_output = array("buff"=>"", "category_srl_list"=>array());

			// 변수 정리
			$category_title_str = sprintf('$_category_title[%d] = "%s"; %s', $node->category_srl, $node->title, $child_output['category_title_str']);

			// 현재 노드의 url값이 공란이 아니라면 category_srl_list 배열값에 입력
			$child_output['category_srl_list'][] = $node->category_srl;
			$output['category_srl_list'] = array_merge($output['category_srl_list'], $child_output['category_srl_list']);

			// node->group_srls값이 있으면
			if($node->group_srls) $group_check_code = sprintf('($is_admin==true||(is_array($group_srls)&&count(array_intersect($group_srls, array(%s)))))',$node->group_srls);
			else $group_check_code = "true";

			// 변수 정리
			$selected = '"'.implode('","',$child_output['category_srl_list']).'"';
			$child_buff = $child_output['buff'];
			$expand = $node->expand;

			// 속성을 생성한다 ( category_srl_list를 이용해서 선택된 메뉴의 노드에 속하는지를 검사한다. 꽁수지만 빠르고 강력하다고 생각;;)
			$attribute = sprintf(
				'"mid" => "%s", "target_srl" => "%d", "node_srl"=>"%s", "category_srl"=>"%s", "parent_srl"=>"%s", "color"=>"%s", "text"=>$_category_title[%d], "selected"=>(in_array(Context::get("category"), array(%s))?1:0), "expand"=>"%s", "list"=>array(%s), "document_count"=>"%d", "grant"=>%s?true:false',
				$node->mid,
				$node->target_srl,
				$node->category_srl,
				$node->category_srl,
				$node->parent_srl,
				$node->color,
				$node->category_srl,
				$selected,
				$expand,
				$child_buff,
				$node->document_count,
				$group_check_code
			);

			// buff 데이터를 생성한다
			$output['buff'] .=  sprintf('%s=>array(%s),', $node->category_srl, $attribute);
			$output['category_title_str'] .= $category_title_str;
		}
		return $output;
	}


}
?>